package AltiConsole;
//This class:
// - Starts up the communication with the altimeter.
// - Reads the data coming in from the altimeter and
//   converts that data in to a useful form.
// - Closes communication with the altimeter.


//Load Libraries
import java.awt.Color;
import java.io.*;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.TooManyListenersException;
//Load RXTX Library
import gnu.io.*;

class AltimeterSerial implements SerialPortEventListener
{
	//passed from main GUI
	AltiConsoleMainScreen window = null;
	
	//////////////////////////////////////
	// variable to return the result of the command
	//////////////////////////////////////
	
	//Store number of flight
	public int NbrOfFlight = 0;
	public int currentFlightNbr = 0;
	public FlightData MyFlight = null;
	public boolean DataReady = false;
	

	/////////////////
    //for containing the ports that will be found
    private Enumeration ports = null;
    //map the port names to CommPortIdentifiers
    private HashMap portMap = new HashMap();

    //this is the object that contains the opened port
    private CommPortIdentifier selectedPortIdentifier = null;
    private SerialPort serialPort = null;

    //input and output streams for sending and receiving data
    private InputStream input = null;
    private OutputStream output = null;

    //just a boolean flag that i use for enabling
    //and disabling buttons depending on whether the program
    //is connected to a serial port or not
    private boolean bConnected = false;

    //the timeout value for connecting with the port
    final static int TIMEOUT = 2000;
    
    //a string for recording what goes on in the program
    //this string is written to the GUI
    String logText = "";
 
  
   public AltimeterSerial(AltiConsoleMainScreen window)
   {
       this.window = window;
   }
 
   //search for all the serial ports
   //pre: none
   //post: adds all the found ports to a combo box on the GUI
   public void searchForPorts()
   {
       ports = CommPortIdentifier.getPortIdentifiers();

       while (ports.hasMoreElements())
       {
           CommPortIdentifier curPort = (CommPortIdentifier)ports.nextElement();

           window.comPorts.addItem(curPort.getName());
           portMap.put(curPort.getName(), curPort);
           
       }
   }
   
   
   public void connect()
   {
       String selectedPort = (String)window.comPorts.getSelectedItem();
       selectedPortIdentifier = (CommPortIdentifier)portMap.get(selectedPort);
       System.out.println("connecting ....\n");
       CommPort commPort = null;
       MyFlight = new FlightData();
       try
       {
           //the method below returns an object of type CommPort
           commPort = selectedPortIdentifier.open("TigerControlPanel", TIMEOUT);
           //the CommPort object can be casted to a SerialPort object
           serialPort = (SerialPort)commPort;
           System.out.println("Speed: " +(String)window.serialRates.getSelectedItem());
           serialPort.setSerialPortParams( Integer.parseInt((String)window.serialRates.getSelectedItem()),
        		   serialPort.DATABITS_8,
        		   serialPort.STOPBITS_1,
        		   serialPort.PARITY_NONE);
        
           //for controlling GUI elements
           setConnected(true);


           //logging
           logText = selectedPort + " opened successfully.";
           window.txtLog.setForeground(Color.black);
           window.txtLog.append(logText + "\n");

       }
       catch (PortInUseException e)
       {
           logText = selectedPort + " is in use. (" + e.toString() + ")";
           
           window.txtLog.setForeground(Color.RED);
           window.txtLog.append(logText + "\n");
       }
       catch (Exception e)
       {
           logText = "Failed to open " + selectedPort + "(" + e.toString() + ")";
           window.txtLog.append(logText + "\n");
           window.txtLog.setForeground(Color.RED);
       }
   }
   
   //open the input and output streams
   //pre: an open port
   //post: initialized input and output streams for use to communicate data
   public boolean initIOStream()
   {
       //return value for weather opening the streams is successful or not
       boolean successful = false;

       try {
           
           input = serialPort.getInputStream();
           output = serialPort.getOutputStream();
           //writeData(0, 0);
           
           successful = true;
           return successful;
       }
       catch (IOException e) {
           logText = "I/O Streams failed to open. (" + e.toString() + ")";
           window.txtLog.setForeground(Color.red);
           window.txtLog.append(logText + "\n");
           return successful;
       }
   }

   //starts the event listener that knows whenever data is available to be read
   //pre: an open serial port
   //post: an event listener for the serial port that knows when data is recieved
   public boolean initListener()
   {
	 //return value for whether opening the initListener is successful or not
       boolean successful = false;
       try
       {
           serialPort.addEventListener(this);
           serialPort.notifyOnDataAvailable(true);
           successful = true;
           return successful;
       }
       catch (TooManyListenersException e)
       {
           logText = "Too many listeners. (" + e.toString() + ")";
           window.txtLog.setForeground(Color.red);
           window.txtLog.append(logText + "\n");
           return successful;
       }
   }

   //disconnect the serial port
   //pre: an open serial port
   //post: closed serial port
   public void disconnect()
   {
	   class CloseThread extends Thread
	   {
	            public void run()
	            {
	           	 serialPort.removeEventListener();
	           	 serialPort.close();
	            }
	    }
       //close the serial port
       try
       {
           writeData(0, 0);
           
           System.out.println("write data ok\n");
           
           output.flush();
           
           input.close();
           System.out.println("input closed\n");
           output.close();
           System.out.println("output closed\n");
           
           serialPort.removeEventListener();
           System.out.println("Listener removed\n");
           
           serialPort.close();
           System.out.println("port closed\n");
           serialPort =null;
           
           setConnected(false);
           
           System.out.println("Sucessfully disconnected\n");
           logText = "Disconnected.";
           window.txtLog.setForeground(Color.red);
           window.txtLog.append(logText + "\n");
       }
       catch (Exception e)
       {
    	   System.out.println("Error disconnected\n");
           logText = "Failed to close " + serialPort.getName() + "(" + e.toString() + ")";
           window.txtLog.setForeground(Color.red);
           window.txtLog.append(logText + "\n");
       }
   }

   final public boolean getConnected()
   {
       return bConnected;
   }
   public void setConnected(boolean bConnected)
   {
       this.bConnected = bConnected;
   }
   
   //method that can be called to send data
   //pre: open serial port
   //post: data sent to the other device
   public void writeData(int leftThrottle, int rightThrottle)
   {
       try
       {
           output.write(leftThrottle);
           
           output.flush();
           
           output.write(rightThrottle);
           output.flush();
          
       }
       catch (Exception e)
       {
           logText = "Failed to write data. (" + e.toString() + ")";
           window.txtLog.setForeground(Color.red);
           window.txtLog.append(logText + "\n");
       }
   }
   
   public void writeData(String command)
   {
       try
       {
      
       	window.txtLog.append( "trying new func\n");
       	
       	window.txtLog.append(command);
       	output.write(command.getBytes());
       	output.flush();

       }
       catch (Exception e)
       {
           logText = "Failed to write data. (" + e.toString() + ")";
           window.txtLog.setForeground(Color.red);
           window.txtLog.append(logText + "\n");
       }
   }

   
   public Sentence readSentence (String tempBuff)
   {
		  //we have a sentence let's find out what it is
		  String tempArray[]=tempBuff.split(",");
		  Sentence sentence = new Sentence();
		  sentence.keyword = tempArray[0];
		 
		  if (tempArray.length > 1)
			  sentence.value1 = Integer.parseInt(tempArray[1]);
		  
		  if (tempArray.length > 2)
			  sentence.value2 = Integer.parseInt(tempArray[2]);
		  
		  if (tempArray.length > 3)
			  sentence.value3 = Integer.parseInt(tempArray[3]);
	   return sentence;
   }

  
   //Reads the incoming data packets from altimeter. 
   public void serialEvent(SerialPortEvent event) 
   { 

      //Reads in data while data is available
      while (event.getEventType()== SerialPortEvent.DATA_AVAILABLE ) 
      {
    	 
         try 
         {
        	 while (input.available() > 0) {
           
            //Read in the available character
            char ch = (char)input.read();

               if (ch=='$')
               {
            	   
            	  //read entire sentence until the end
            	  String tempBuff="";
            	  while (ch !=';')
             	  {
             		  //this is not the end of our command
             		  ch = (char)input.read();
             		 
             		 if (ch!='\n')
             			 if(ch!=';')
             				 tempBuff = tempBuff + Character.toString(ch);
             	  }
            	 
            	  Sentence currentSentence = null;
            	  if (!tempBuff.isEmpty())
            	  {
            		  currentSentence = readSentence(tempBuff);  
            		  System.out.println(currentSentence.keyword);
            	  }
            	  
            	  if (currentSentence.keyword.equals("data")  ) 
            	  {
            		  //Value 1 contain the flight number
            		  currentFlightNbr = (int) currentSentence.value1;
            		  //Value 2 contain the time
            		  //Value 3 contain the altitude
            		  MyFlight.AddToFlight(currentSentence.value2, 
            				  currentSentence.value3, 
            				  "Flight "+ currentFlightNbr);
            		  System.out.println("Flight " + currentFlightNbr);
            		  System.out.println("X=" +currentSentence.value2);
            		  System.out.println("Y=" +currentSentence.value3);
            	  } 
            	  else if(currentSentence.keyword.equals( "nbrOfFlight"))
            	  {
            		//Value 1 contains the number of flight
            		  NbrOfFlight = (int)currentSentence.value1;
            	  }
            	  else if(currentSentence.keyword.equals( "start"))
            	  {
            		//We are starting reading data
            		  DataReady =false;
            	  }
            	  else if(currentSentence.keyword.equals( "end"))
            	  {
            		//We have finished reading data
            		  DataReady =true;
            	  }
               }
        	 }
         } 
         catch (IOException e) 
         {
         }
         
      }
   }

}

class Sentence 
{

	public String keyword;
	public long value1;
	public long value2;
	public long value3;
}


